package com.jeeplus.test.finance.invoice.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jeeplus.aop.logging.annotation.ApiLog;
import com.jeeplus.sys.constant.enums.LogTypeEnum;
import com.jeeplus.sys.utils.StringUtils;
import com.jeeplus.test.finance.invoice.domain.FinanceInvoice;
import com.jeeplus.test.finance.invoice.service.FinanceInvoiceService;
import com.jeeplus.test.finance.invoice.service.dto.FinanceInvoiceDTO;
import com.jeeplus.test.program.configuration.fileDict.domain.ProgramFileDict;
import com.jeeplus.test.program.configuration.fileDict.service.ProgramFileDictService;
import com.jeeplus.test.program.configuration.fileDict.service.dto.ProgramFileDictDTO;
import io.swagger.annotations.Api;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

@Api("财务管理-发票")
@RestController
@RequestMapping(value = "/finance/invoice")
public class FinanceInvoiceController {

    @Resource
    private FinanceInvoiceService financeInvoiceService;

    /**
     * 查询发票列表
     * @param financeInvoiceDTO
     * @param page
     * @return
     * @throws Exception
     */
    @ApiLog("查询发票列表")
    @PreAuthorize("hasAuthority('finance:invoice:list')")
    @GetMapping("list")
    public ResponseEntity<IPage<FinanceInvoiceDTO>> data(FinanceInvoiceDTO financeInvoiceDTO, Page<FinanceInvoiceDTO> page) {
        IPage<FinanceInvoiceDTO> result = new Page<FinanceInvoiceDTO>();
        result = financeInvoiceService.findList (page,new QueryWrapper<FinanceInvoice>());
        return ResponseEntity.ok (result);
    }

    /**
     * 查询发票数据
     * @param id
     * @return
     */
    @ApiLog("查询发票数据")
    @PreAuthorize ("hasAnyAuthority('finance:invoice:view','finance:invoice:add','finance:invoice:edit')")
    @GetMapping("queryById")
    public ResponseEntity queryById(@RequestParam("id") String id) {
        FinanceInvoiceDTO financeInvoiceDTO = financeInvoiceService.queryById ( id );
        return ResponseEntity.ok (financeInvoiceDTO);
    }

    /**
     * 保存发票数据
     * @param financeInvoiceDTO
     * @return
     */
    @ApiLog(value = "修改/新增发票数据", type = LogTypeEnum.SAVE)
    @PreAuthorize("hasAnyAuthority('finance:invoice:add','finance:invoice:edit')")
    @PostMapping("save")
    public ResponseEntity save(@Valid @RequestBody FinanceInvoiceDTO financeInvoiceDTO) throws Exception {
        financeInvoiceService.saveInvoice(financeInvoiceDTO);
        return ResponseEntity.ok ("操作成功");
    }

    /**
     * 删除附件类型
     * @param ids
     * @return
     */
    @ApiLog(value = "删除附件类型", type = LogTypeEnum.SAVE)
    @PreAuthorize ("hasAuthority('finance:invoice:del')")
    @DeleteMapping("delete")
    public ResponseEntity delete(String ids) {
        return financeInvoiceService.deleteByIds(ids);
    }
}
