package com.jeeplus.test.finance.invoice.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.jeeplus.sys.service.dto.UserDTO;
import com.jeeplus.sys.utils.StringUtils;
import com.jeeplus.sys.utils.UserUtils;
import com.jeeplus.test.finance.invoice.domain.FinanceInvoice;
import com.jeeplus.test.finance.invoice.domain.FinanceInvoiceBase;
import com.jeeplus.test.finance.invoice.mapper.FinanceInvoiceBaseMapper;
import com.jeeplus.test.finance.invoice.mapper.FinanceInvoiceMapper;
import com.jeeplus.test.finance.invoice.service.dto.FinanceInvoiceBaseDTO;
import com.jeeplus.test.finance.invoice.service.dto.FinanceInvoiceDTO;
import com.jeeplus.test.finance.invoice.service.mapstruct.FinanceInvoiceBaseWrapper;
import com.jeeplus.test.finance.invoice.service.mapstruct.FinanceInvoiceWrapper;
import com.jeeplus.test.mould.service.SerialnumTplService;
import com.jeeplus.test.program.configuration.fileDict.domain.ProgramFileDict;
import com.jeeplus.test.program.configuration.fileDict.mapper.ProgramFileDictMapper;
import com.jeeplus.test.program.configuration.fileDict.service.dto.ProgramFileDictDTO;
import com.jeeplus.test.program.configuration.fileDict.service.mapstruct.ProgramFileDictWrapper;
import com.jeeplus.test.roster.service.dto.RosterBaseDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Transactional
public class FinanceInvoiceService extends ServiceImpl<FinanceInvoiceMapper, FinanceInvoice> {

    @Resource
    private FinanceInvoiceMapper financeInvoiceMapper;
    @Resource
    private FinanceInvoiceBaseMapper financeInvoiceBaseMapper;
    @Resource
    private SerialnumTplService serialnumTplService;

    public IPage<FinanceInvoiceDTO> findList(Page<FinanceInvoiceDTO> page, QueryWrapper<FinanceInvoice> queryWrapper){
        queryWrapper.eq("fi.del_flag","0");
        return financeInvoiceMapper.findList(page,queryWrapper);
    }

    public FinanceInvoiceDTO queryById(String id) {

        FinanceInvoiceDTO financeInvoiceDTO = financeInvoiceMapper.queryById(id);

        return financeInvoiceDTO;
    }

    public ResponseEntity saveInvoice(FinanceInvoiceDTO financeInvoiceDTO) throws Exception{
        FinanceInvoice financeInvoice = FinanceInvoiceWrapper.INSTANCE.toEntity(financeInvoiceDTO);
        if (ObjectUtil.isNotEmpty(financeInvoice)) {
            if(StringUtils.isBlank(financeInvoice.getId())){
                //获取当前登录人信息
                UserDTO userDTO = UserUtils.getCurrentUserDTO();
                //发票编号生成
                String serialNum = serialnumTplService.genSerialNum(userDTO.getCompanyDTO().getId(), FinanceInvoiceDTO.BIZ_CODE);
                financeInvoice.setNo(serialNum);
            }
        }
        this.saveOrUpdate(financeInvoice);
        if (ObjectUtil.isNotEmpty(financeInvoiceDTO)) {
            if(CollectionUtil.isNotEmpty(financeInvoiceDTO.getFinanceInvoiceBaseDTOList())){
                List<String> delIds = financeInvoiceDTO.getFinanceInvoiceBaseDTOList().stream().distinct().filter(item->{
                    if(StringUtils.isNotBlank(item.getId())){
                        return true;
                    }
                    return false;
                }).map(FinanceInvoiceBaseDTO::getId).collect(Collectors.toList());
                if(CollectionUtil.isNotEmpty(delIds)){
                    financeInvoiceBaseMapper.delete(new QueryWrapper<FinanceInvoiceBase>().lambda()
                            .eq(FinanceInvoiceBase::getInvoiceId,financeInvoice.getId()).notIn(FinanceInvoiceBase::getId,delIds));
                }else{
                    financeInvoiceBaseMapper.delete(new QueryWrapper<FinanceInvoiceBase>().lambda()
                            .eq(FinanceInvoiceBase::getInvoiceId,financeInvoice.getId()));
                }
                financeInvoiceDTO.getFinanceInvoiceBaseDTOList().stream().forEach(item->{
                    FinanceInvoiceBase financeInvoiceBase = FinanceInvoiceBaseWrapper.INSTANCE.toEntity(item);
                    if(StringUtils.isNotBlank(financeInvoiceBase.getId())){
                        financeInvoiceBaseMapper.updateById(financeInvoiceBase);
                    }else{
                        financeInvoiceBase.setInvoiceId(financeInvoice.getId());
                        financeInvoiceBaseMapper.insert(financeInvoiceBase);
                    }
                });
            }else{
                financeInvoiceBaseMapper.delete(new QueryWrapper<FinanceInvoiceBase>().lambda()
                        .eq(FinanceInvoiceBase::getInvoiceId,financeInvoice.getId()));
            }
        }

        return ResponseEntity.ok("保存成功");
    }

    public ResponseEntity deleteByIds(String ids) {
        String idArray[] =ids.split(",");
        this.removeByIds (Lists.newArrayList (idArray));
        return ResponseEntity.ok ("删除成功");
    }
}
